// Functions for sorting and ordering transitions

#include <stdio.h>
#include "nummrit.h"
#include "maxvalue.h"
#include "sort_util.h"
#include <math.h>

//extern CString sim_out;

//////////////////////////////////////////////////////////////////////////////////
// sortutil.cpp
// assorted transition sorting routines used by SpinWorks
//////////////////////////////////////////////////////////////////////////////////

void killdupe(const char *sim_out, int maxtran, int ntrans, int *lineno, double *freq, 
			 double *weight, double *calc, double *diff)
{
	// kill any duplicate assignments
	double killfrq;
	short killed = FALSE;
	int curr; 
	int oldtrn = ntrans;
	int killno = -1;

	for(curr = 1; curr <= oldtrn; curr++)
	{
		if(lineno[curr] == killno)
		{
			FILE *ofp = fopen(sim_out, "a");
			if(!killed)
			{
				fprintf(ofp, "\n***  Ignored duplicate transition %8d with freq %11.3f\n",
					killno, killfrq);
				killed = TRUE;
			}
			fprintf(ofp, "\n***  Ignored duplicate transition %8d with freq %11.3f\n",
				killno, freq[curr]);
			if(curr == oldtrn) ntrans--;
			fclose(ofp);
		}
		else
		{
			killno = lineno[curr];
			killfrq = freq[curr];
			if(!killed) ntrans++;
			lineno[ntrans] = lineno[curr];
			freq[ntrans] = freq[curr];
			weight[ntrans] = weight[curr];
			calc[ntrans] = calc[curr];
			diff[ntrans] = diff[curr];
			killed = FALSE;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////

void stobsf(int maxtrn, int ntrans, int *line_no, double *freq, double *weight)
{
	//order observed transitions  in increasing frequency order
	const double	alog2 = 0.3010299956639812;
	double	hldfrq, hldwgt;
	short	found;
	int		hldlin, posn, dist, distp1, i, newpos, log2;

	if (ntrans > 0)
	{
	//	pass through increments
		log2 = (int)(log10((double)ntrans)/alog2);	// check!
		dist = (int)pow((double)2, log2) - 1;

		while(dist > 0)
		{
			distp1 = dist + 1;

			for(i = distp1; i <= ntrans; i++)
			{
				hldlin = line_no[i];
				hldfrq = freq[i];
				hldwgt = weight[i];

				posn = i - dist;
				found = FALSE;

				while(posn >= 1 && !found) 
				{
					if(hldfrq >= freq[posn])
					{
						found = TRUE;
					}
					else
					{
						newpos = posn + dist;
						line_no[newpos] = line_no[posn];
						freq[newpos] = freq[posn];
						weight[newpos] = weight[posn];
						posn = posn - dist;
					}
				}

				newpos = posn + dist;
				line_no[newpos] = hldlin;
				freq[newpos] = hldfrq;
				weight[newpos] = hldwgt;
			}
			dist = dist/2;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////

void stasnl(int maxtrn, int ntrans, int *line_no, double *freq, 
			double *weight, double *calc, double *diff)
{
	//order assigned in increasing line number order
	double	alog2 = 0.3010299956639812;
	double	hldfrq, hldwgt, hldclc, hlddif;
	short	found;
	int		hldlin, posn, dist, distp1, i, newpos, log2;

	if (ntrans > 0)
	{
	//	pass through increments
		log2 = (int)(log10((double)ntrans)/alog2);	// check!
		dist = (int)pow((double)2, log2) - 1;

		while(dist > 0)
		{
			distp1 = dist + 1;

			for(i = distp1; i <= ntrans; i++)
			{
				hldlin = line_no[i];
				hldfrq = freq[i];
				hldwgt = weight[i];
				hldclc = calc[i];
				hlddif = diff[i];

				posn = i - dist;
				found = FALSE;

				while(posn >= 1 && !found) 
				{
					if(hldlin >= line_no[posn])
					{
						found = TRUE;
					}
					else
					{
						newpos = posn + dist;
						line_no[newpos] = line_no[posn];
						freq[newpos] = freq[posn];
						weight[newpos] = weight[posn];
						calc[newpos] = calc[posn];
						diff[newpos] = diff[posn];
						posn = posn - dist;
					}
				}

				newpos = posn + dist;
				line_no[newpos] = hldlin;
				freq[newpos] = hldfrq;
				weight[newpos] = hldwgt;
				calc[newpos] = hldclc;
				diff[newpos] = hlddif;
			}
			dist = dist/2;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////

// use a Hibbard modified shell sort to order the transitions by frequency

void shell_sort_trans(int maxtrn, int clc_count, double *freq, double *ampl,
				int *line_no, int *tran_key)
{
	double	alog2 = 0.3010299956639812;
	double	hldfrq, hldamp;
	short	found;
	int		hldkey, hldlin, posn, dist, distp1, i, newpos, log2;

	if (clc_count > 0)
	{
	//	pass through increments
		log2 = (int)(log10((double)clc_count)/alog2);	// check!
		dist = (int)pow((double)2, log2) - 1;

		while(dist > 0)
		{
			distp1 = dist + 1;

			for(i = distp1; i <= clc_count; i++)
			{
				hldlin = line_no[i];
				hldfrq = freq[i];
				hldamp = ampl[i];
				hldkey = tran_key[i];

				posn = i - dist;
				found = FALSE;

				while(posn >= 1 && !found) 
				{

					if(hldfrq >= freq[posn])
					{
						found = TRUE;
					}
					else
					{
						newpos = posn + dist;
						line_no[newpos] = line_no[posn];
						freq[newpos] = freq[posn];
						ampl[newpos] = ampl[posn];
						tran_key[newpos] = tran_key[posn];
						posn = posn - dist;
					}
				}

				newpos = posn + dist;
				line_no[newpos] = hldlin;
				freq[newpos] = hldfrq;
				ampl[newpos] = hldamp;
				tran_key[newpos] = hldkey;
			}
			dist = dist/2;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////////

// use a Hibbard modified shell sort to order the assigned transitions 
// by frequency

void shell_sort_asn(int maxasn, int ntrans, int *line_no, double *freq,
				 double *weight, double *calc, double *diff)
{
	double	alog2 = 0.3010299956639812;
	double	hldfrq, hldwgt, hldclc, hlddif;
	short	found;
	int		hldlin, posn, dist, distp1, i, newpos, log2;

	if (ntrans > 0)
	{
	//	pass through increments
		log2 = (int)(log10((double)ntrans)/alog2);	// check!
		dist = (int)pow((double)2, log2) - 1;

		while(dist > 0)
		{
			distp1 = dist + 1;

			for(i = distp1; i <= ntrans; i++)
			{
				hldlin = line_no[i];
				hldfrq = freq[i];
				hldwgt = weight[i];
				hldclc = calc[i];
				hlddif = diff[i];

				posn = i - dist;
				found = FALSE;

				while(posn >= 1 && !found) 
				{
					if(hldfrq >= freq[posn])
					{
						found = TRUE;
					}
					else
					{
						newpos = posn + dist;
						line_no[newpos] = line_no[posn];
						freq[newpos] = freq[posn];
						weight[newpos] = weight[posn];
						calc[newpos] = calc[posn];
						diff[newpos] = diff[posn];
						posn = posn - dist;
					}
				}

				newpos = posn + dist;
				line_no[newpos] = hldlin;
				freq[newpos] = hldfrq;
				weight[newpos] = hldwgt;
				calc[newpos] = hldclc;
				diff[newpos] = hlddif;
			}
			dist = dist/2;
		}
	}
}

